@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from typing import Awaitable, Callable, Dict, List, Optional, Tuple, Union
 
 from aioquic.h3.connection import H3Connection
@@ -12,6 +14,7 @@ from .events import (
     EndBody,
     EndData,
     Event as StreamEvent,
+    InformationalResponse,
     Request,
     Response,
     StreamClosed,
@@ -19,33 +22,41 @@ from .events import (
 from .http_stream import HTTPStream
 from .ws_stream import WSStream
 from ..config import Config
+from ..typing import AppWrapper, TaskGroup, WorkerContext
 from ..utils import filter_pseudo_headers
 
 
 class H3Protocol:
     def __init__(
         self,
+        app: AppWrapper,
         config: Config,
+        context: WorkerContext,
+        task_group: TaskGroup,
         client: Optional[Tuple[str, int]],
         server: Optional[Tuple[str, int]],
-        spawn_app: Callable[[dict, Callable], Awaitable[Callable]],
         quic: QuicConnection,
         send: Callable[[], Awaitable[None]],
     ) -> None:
+        self.app = app
         self.client = client
         self.config = config
+        self.context = context
         self.connection = H3Connection(quic)
         self.send = send
         self.server = server
-        self.spawn_app = spawn_app
         self.streams: Dict[int, Union[HTTPStream, WSStream]] = {}
+        self.task_group = task_group
 
     async def handle(self, quic_event: QuicEvent) -> None:
         for event in self.connection.handle_event(quic_event):
             if isinstance(event, HeadersReceived):
-                await self._create_stream(event)
-                if event.stream_ended:
-                    await self.streams[event.stream_id].handle(EndBody(stream_id=event.stream_id))
+                if not self.context.terminated.is_set():
+                    await self._create_stream(event)
+                    if event.stream_ended:
+                        await self.streams[event.stream_id].handle(
+                            EndBody(stream_id=event.stream_id)
+                        )
             elif isinstance(event, DataReceived):
                 await self.streams[event.stream_id].handle(
                     Body(stream_id=event.stream_id, data=event.data)
@@ -54,7 +65,7 @@ class H3Protocol:
                     await self.streams[event.stream_id].handle(EndBody(stream_id=event.stream_id))
 
     async def stream_send(self, event: StreamEvent) -> None:
-        if isinstance(event, Response):
+        if isinstance(event, (InformationalResponse, Response)):
             self.connection.send_headers(
                 event.stream_id,
                 [(b":status", b"%d" % event.status_code)]
@@ -82,22 +93,26 @@ class H3Protocol:
 
         if method == "CONNECT":
             self.streams[request.stream_id] = WSStream(
+                self.app,
                 self.config,
+                self.context,
+                self.task_group,
                 True,
                 self.client,
                 self.server,
                 self.stream_send,
-                self.spawn_app,
                 request.stream_id,
             )
         else:
             self.streams[request.stream_id] = HTTPStream(
+                self.app,
                 self.config,
+                self.context,
+                self.task_group,
                 True,
                 self.client,
                 self.server,
                 self.stream_send,
-                self.spawn_app,
                 request.stream_id,
             )
 
@@ -110,6 +125,7 @@ class H3Protocol:
                 raw_path=raw_path,
             )
         )
+        await self.context.mark_request()
 
     async def _create_server_push(
         self, stream_id: int, path: bytes, headers: List[Tuple[bytes, bytes]]
