@@ -18,7 +18,7 @@ from wsproto.events import (
 from wsproto.extensions import Extension, PerMessageDeflate
 from wsproto.frame_protocol import CloseReason
 from wsproto.handshake import server_extensions_handshake, WEBSOCKET_VERSION
-from wsproto.utilities import generate_accept_token, split_comma_header
+from wsproto.utilities import generate_accept_token, LocalProtocolError, split_comma_header
 
 from .events import Body, Data, EndBody, EndData, Event, Request, Response, StreamClosed
 from ..config import Config
@@ -231,7 +231,7 @@ class WSStream:
                 self.app_put = await self.task_group.spawn_app(
                     self.app, self.config, self.scope, self.app_send
                 )
-                await self.app_put({"type": "websocket.connect"})  # type: ignore
+                await self.app_put({"type": "websocket.connect"})
         elif isinstance(event, (Body, Data)):
             self.connection.receive_data(event.data)
             await self._handle_events()
@@ -257,7 +257,7 @@ class WSStream:
                     self.scope, {"status": 500, "headers": []}, time() - self.start_time
                 )
             elif self.state == ASGIWebsocketState.CONNECTED:
-                await self._send_wsproto_event(CloseConnection(code=CloseReason.ABNORMAL_CLOSURE))
+                await self._send_wsproto_event(CloseConnection(code=CloseReason.INTERNAL_ERROR))
             await self.send(StreamClosed(stream_id=self.stream_id))
         else:
             if message["type"] == "websocket.accept" and self.state == ASGIWebsocketState.HANDSHAKE:
@@ -333,8 +333,12 @@ class WSStream:
         )
 
     async def _send_wsproto_event(self, event: WSProtoEvent) -> None:
-        data = self.connection.send(event)
-        await self.send(Data(stream_id=self.stream_id, data=data))
+        try:
+            data = self.connection.send(event)
+        except LocalProtocolError:
+            pass
+        else:
+            await self.send(Data(stream_id=self.stream_id, data=data))
 
     async def _accept(self, message: WebsocketAcceptEvent) -> None:
         self.state = ASGIWebsocketState.CONNECTED
