@@ -2,17 +2,24 @@ from __future__ import annotations
 
 from multiprocessing.synchronize import Event as EventType
 from types import TracebackType
-from typing import Any, Awaitable, Callable, Dict, Iterable, Optional, Tuple, Type, Union
+from typing import (
+    Any,
+    Awaitable,
+    Callable,
+    Dict,
+    Iterable,
+    Literal,
+    Optional,
+    Protocol,
+    Tuple,
+    Type,
+    TypedDict,
+    Union,
+)
 
 import h2.events
 import h11
 
-# Till PEP 544 is accepted
-try:
-    from typing import Literal, Protocol, TypedDict
-except ImportError:
-    from typing_extensions import Literal, Protocol, TypedDict  # type: ignore
-
 from .config import Config, Sockets
 
 H11SendableEvent = Union[h11.Data, h11.EndOfMessage, h11.InformationalResponse, h11.Response]
@@ -283,8 +290,12 @@ class Event(Protocol):
 
 class WorkerContext(Protocol):
     event_class: Type[Event]
+    terminate: Event
     terminated: Event
 
+    async def mark_request(self) -> None:
+        ...
+
     @staticmethod
     async def sleep(wait: Union[float, int]) -> None:
         ...
